#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
# This file is part of Rok.
#
# Copyright © 2022 Arrikto Inc.  All Rights Reserved.

"""Helper script to delete the certiticate of a Rok Registry application."""


import os
import sys
import django

from django.conf import settings
from django.db import transaction
from django.utils.timezone import now

from rok_tasks import frontend, question
from rok_tasks.frontend.cli import RokCLI


fr = frontend.get_frontend()

TITLE = "Delete Certificate of Rok Registry Application"

# Questions
######################################################


class ClientIDQuestion(question.LineInputQuestion):
    """Question for Client ID."""

    title = TITLE + ": Client ID"
    name = "question/client_id"
    msg = "What is the client ID of the Rok Registry application?"
    desc = "The client ID of the Rok Registry application."
    envvars = "ROK_REGISTRY_CLIENT_ID"
    argument_opts = {
        "name": ["--client-id"],
        "metavar": "CLIENT_ID",
        "help": desc
    }
    required = True
    allow_empty = False


class ConfirmDeletionQuestion(question.TrueOrFalseQuestion):
    """Question to confirm deleting the certificate."""

    title = TITLE + ": Confirm"
    name = "question/confirm_deletion"
    desc = ("Are you sure you want to delete the certificate of the following"
            " application?")
    envvars = "ROK_REGISTRY_DELETE_CERT"
    default = False
    true = frontend.Choice(id="true", desc="Delete certificate")
    false = frontend.Choice(id="false", desc="Exit")
    no_cancel = True
    no_help = True
    argument_opts = {
        "name": ["--confirm-deletion"],
        "help": "Delete the certificate of the provided application."
    }
    required = True
    allow_empty = False

    def __init__(self, client_id, **kwargs):
        msg = "\n" + client_id
        super(ConfirmDeletionQuestion, self).__init__(msg=msg, **kwargs)

# CLI
######################################################


class CertDeleteCLI(RokCLI):
    """CLI tool to delete the certificate of a Rok Registry application."""

    DESCRIPTION = "Delete the certificate of a Rok Registry application."""

    DEFAULT_FRONTEND = "readline"
    DEFAULT_LOGFILE = "indexer-cert-delete.log"

    def initialize_parser(self):
        parser = super(CertDeleteCLI, self).initialize_parser()
        questions = [ClientIDQuestion(),
                     ConfirmDeletionQuestion(client_id="")]
        question.add_questions_args(questions, parser)
        return parser

    def ask_questions(self):
        questions = [ClientIDQuestion()]
        for q in questions:
            self.qctx.add_question(q)
            q.ask()

    @classmethod
    def delete_cert(cls, client_id):
        # We make this import here instead of top-level because django.setup()
        # must have been called previously.
        #
        # See also:
        # https://docs.djangoproject.com/en/2.1/topics/settings/#calling-django-setup-is-required-for-standalone-django-usage  # noqa: E501
        from rok_indexer_api.logic.apps import get_app
        with transaction.atomic():
            app = get_app(client_id, lock=True)
            app.certificate = ""
            app.certificate_fingerprint = ""
            app.certificate_created_at = now()
            app.save()

    def main(self):
        """Delete the certificate of a Rok Registry application."""
        self.ask_questions()
        client_id = self.qctx.questions[ClientIDQuestion.name].get_answer()
        confirm_q = ConfirmDeletionQuestion(client_id=client_id)
        self.qctx.add_question(confirm_q)
        if not confirm_q.ask():
            raise frontend.UserCanceledInputError

        with fr.Progress(maxval=1, title=TITLE) as p:
            p.info("Deleting the certificate of the Rok Registry"
                   " application...")
            self.delete_cert(client_id)
            p.success("Successfully deleted the certificate of the Rok"
                      " Registry application with client ID `%s'." % client_id)


def main():
    # https://docs.djangoproject.com/en/2.1/topics/settings/#either-configure-or-django-settings-module-is-required  # noqa: E501
    if not settings.configured and "DJANGO_SETTINGS_MODULE" not in os.environ:
        os.environ.setdefault("DJANGO_SETTINGS_MODULE",
                              "rok_indexer_prj.settings")
    django.setup()

    cli = CertDeleteCLI()
    return cli.run()


if __name__ == "__main__":
    sys.exit(main())
